#include "stdafx.h"
#include "resource.h"

#include "bitmap.h"
#include "ff3se.h"
#include "tab1.h"
#include "monster.h"
#include "monsters.h"
#include "options.h"
#include "export.h"

Tab1 tab1;

/*************************************/
//Destructor for Tab1
Tab1::~Tab1(void)
{
	DeleteDC(monsHdc);
	DeleteDC(editHdc);

	DeleteObject(monsBMP);
	DeleteObject(editBMP);
}

/*************************************/
//Adds the list of names to the combo box
//if on tab1, monster names are added, if on tab2 esper names are added
void Tab1::addNames(void)
{
	int i;

	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER, CB_RESETCONTENT, 0, 0);
	if (tabSelected == 0)
	{
		for (i = 0; i < 384; i++)
		{
			SendDlgItemMessage(hTab, IDC_COMBO_MONSTER, CB_ADDSTRING, 0,
				(WPARAM)(LPCSTR)(monsters.getMonster(i))->name);
		}
	}
	else
	{
		for (i = 384; i < 415; i++)
		{
			if (i != 400)			//Don't add Ragnarok
				SendDlgItemMessage(hTab, IDC_COMBO_MONSTER, CB_ADDSTRING, 0,
				(WPARAM)(LPCSTR)(monsters.getMonster(i))->name);
		}
	}

	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER, CB_SETCURSEL, 0, 0);

	if (tabSelected == 0)
		monsSelected = 0;
	else
		monsSelected = 384;
}

/*************************************/
//Handles the action of the brush button
//Sets the pixel x,y of the monster to the palette entry selected
void Tab1::brushButton(int x, int y)
{
	int max;

	monsChanged = true;
	m_changed = true;

	if ((monsters.getMonster(monsSelected))->getSize() == 0)
		max = 64;
	else max = 128;

	if ((x >= max) || (y >= max))
		return;

	(monsters.getMonster(monsSelected))->setPixel(x, y, palEntrySelected);

	invalidateMonsterPics();
}

/*************************************/
//Changes the Monster's image to the image specified in the combo box
void Tab1::changeImage(void)
{
	m_changed = true;

	(monsters.getMonster(monsSelected))->setImage(SendDlgItemMessage(hTab,
		IDC_COMBO_MONSTER_IMAGE, CB_GETCURSEL, 0, 0));

	createImageList();
	invalidateMonsterPics();
}

/*************************************/
//Changes the Number of colors used by the monster
void Tab1::changeNumColors(void)
{
	BYTE num;

	m_changed = true;

	num = (BYTE)SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_COLORS, CB_GETCURSEL, 0, 0);

	if (num == 0)
		num = 8;
	else
		num = 16;

	(monsters.getMonster(monsSelected))->setNumColors(num);

	invalidatePics();
}

/*************************************/
//Changes the palette used by the Monster to the palette selected in the combo box
void Tab1::changePalette(void)
{
	int palSelected;

	m_changed = true;

	palSelected = SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_PALETTE, CB_GETCURSEL, 0, 0);
	(monsters.getMonster(monsSelected))->changePalette(palSelected);

	invalidatePics();
}

/*************************************/
//Changes which palette entry is selected
void Tab1::changePalEntry(BYTE num)
{
	palEntrySelected = num;
	sliderEnable();
	invalidateTabControl(IDC_PIC_MONSTER_PALETTE);
}

/*************************************/
//Changes the Monster's size
void Tab1::changeSize(void)
{
	int size;

	m_changed = true;

	size = SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_SIZE, CB_GETCURSEL, 0, 0);

	(monsters.getMonster(monsSelected))->setSize(size);

	invalidatePics();
}

/*************************************/
//Changes which color is transparent
//Swaps the currently selected color with color 0
void Tab1::changeTransparency(void)
{
	m_changed = true;

	(monsters.getMonster(monsSelected))->setTransparency(palEntrySelected);

	monsChanged = true;

	changePalEntry(0);

	invalidatePics();
}

/*************************************/
//Clears the Monster's image to a blank image
void Tab1::clearImage(void)
{
	if (MessageBox(NULL, "This will erase the current monster's picture", ff3seTitle, MB_OKCANCEL | MB_TASKMODAL) != IDOK)
		return;

	m_changed = true;

	(monsters.getMonster(monsSelected))->clearImage();
	monsChanged = true;
	invalidatePics();
}

/*************************************/
//Copies a different monster's image to this one
void Tab1::copyImage(void)
{
	int i;

	m_changed = true;

	hDlg = CreateDialog(hInst, MAKEINTRESOURCE(IDD_COPY_IMAGE), hTab, ::copyMonsterProc);

	SendDlgItemMessage(hDlg, IDC_COPY_LIST, LB_RESETCONTENT, 0, 0);
	for (i = 0; i < 411; i++)
	{
		SendDlgItemMessage(hDlg, IDC_COPY_LIST, LB_ADDSTRING, 0,
			(LPARAM)(LPCTSTR)(monsters.getMonster(i))->name);
	}

	SetWindowText(hDlg, "Copy monster image");
	BringWindowToTop(hDlg);
	ShowWindow(hDlg, SW_NORMAL);
	EnableWindow(hWnd, false);
}

/*************************************/
//Creates the tab
void Tab1::create(void)
{
	hTab = CreateDialog(hInst, MAKEINTRESOURCE(IDD_TAB1), ::hMainTab, tab1Proc);
	SetWindowPos(hTab, HWND_TOP, 5, 30, 680, 480, 0);
}

/*************************************/
//Updates HDCs for the monster and editing pic with the new bitmap images
void Tab1::createHDCs(void)
{
	int xScroll, yScroll;

	xScroll = SendDlgItemMessage(hTab, IDC_HSCROLL_MONSTER_EDIT, SBM_GETPOS, 0, 0);
	yScroll = SendDlgItemMessage(hTab, IDC_VSCROLL_MONSTER_EDIT, SBM_GETPOS, 0, 0);

	(monsters.getMonster(monsSelected))->pic.drawToDC(monsHdc, &backBmp, 128, 128);

	(monsters.getMonster(monsSelected))->pic.drawToLargeDC(
		magnification, editHdc, xScroll, yScroll, &back2Bmp, 256, 256);
}

/*************************************/
//Creates the list of images and puts it into the combo box
void Tab1::createImageList(void)
{
	int i, j;
	int found;
	char imageName[40];
	char temp[5];

	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_IMAGE, CB_RESETCONTENT, 0, 0);

	for (i = 0; i < monsters.getNumImages(); i++)
	{
		found = -1;
		for (j = 0; (j < 411) && (found == -1); j++)
		{
			if ((monsters.getMonster(j))->getImageNum() == i)
				found = j;
		}
		if (found == -1)
		{
			strcpy(imageName, "Image ");
			strcat(imageName, _itoa(i, temp, 10));
		}
		else
		{
			strcpy(imageName, "Image ");
			strcat(imageName, _itoa(i, temp, 10));
			strcat(imageName, " - ");
			strcat(imageName, (monsters.getMonster(found))->name);
		}
		SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_IMAGE, CB_ADDSTRING, 0,
			(LPARAM)(LPCTSTR)imageName);
	}
}

/*************************************/
//Displays the preview of the file selected in the import dialog box
void Tab1::displayPreview(HWND hdlg)
{
	char filename[200];
	HDC picHdc, bmpHdc;
	HWND hPic, hText;
	Bitmap bmp;
	BYTE result;
	HBITMAP tempBMP;
	int size;

	size = CommDlg_OpenSave_GetSpec(GetParent(hdlg), filename, 200);
	if (size == 1)		//Exit if no file selected
	{
		SendDlgItemMessage(hdlg, IDC_TEXT_COLORS, WM_SETTEXT, (WPARAM)0, (LPARAM)(LPCTSTR)"");
		SendDlgItemMessage(hdlg, IDC_TEXT_SIZE, WM_SETTEXT, (WPARAM)0, (LPARAM)(LPCTSTR)"");
		return;
	}
	CommDlg_OpenSave_GetFilePath(GetParent(hdlg), filename, 200);
	hPic = GetDlgItem(hdlg, IDC_MONSPREVIEW);
	picHdc = GetDC(hPic);							//Create an HDC for the preview picture control

	result = bmp.loadFile(filename, 16, 128, 128);
	if (result != 0)
	{
		backBmp.drawToDC(picHdc);

		//Set text message to display reason image can't be previewed
		SendDlgItemMessage(hdlg, IDC_TEXT_COLORS, WM_SETTEXT, (WPARAM)0, (LPARAM)(LPCTSTR)"");
		if (result == 1)
		{
			SendDlgItemMessage(hdlg, IDC_TEXT_SIZE, WM_SETTEXT, (WPARAM)0,
				(LPARAM)(LPCTSTR)"Image has over 16 colors");
		}
		else if (result == 2)
		{
			SendDlgItemMessage(hdlg, IDC_TEXT_SIZE, WM_SETTEXT, (WPARAM)0,
				(LPARAM)(LPCTSTR)"Image is larger than 128x128");
		}
		else if (result == 3)
		{
			SendDlgItemMessage(hdlg, IDC_TEXT_SIZE, WM_SETTEXT, (WPARAM)0,
				(LPARAM)(LPCTSTR)"Error opening image");
		}
		else if (result == 4)
		{
			SendDlgItemMessage(hdlg, IDC_TEXT_SIZE, WM_SETTEXT, (WPARAM)0,
				(LPARAM)(LPCTSTR)"Not a valid image file");
		}
	}
	else
	{
		bmpHdc = CreateCompatibleDC(screenHdc);
		tempBMP = CreateCompatibleBitmap(screenHdc, 128, 128);
		SelectObject(bmpHdc, tempBMP);
		bmp.drawToDC(bmpHdc, &backBmp, 128, 128);

		BitBlt(picHdc, 0, 0, 128, 128, bmpHdc, 0, 0, SRCCOPY);

		hText = GetDlgItem(hdlg, IDC_TEXT_COLORS);

		//Set text to display number of colors imported monster will use
		if (bmp.numColors() <= 8)
		{
			SendMessage(hText, WM_SETTEXT, (WPARAM)0, (LPARAM)(LPCTSTR)"8 colors");
		}
		else
		{
			SendMessage(hText, WM_SETTEXT, (WPARAM)0, (LPARAM)(LPCTSTR)"16 colors");
		}
		//Set text to display size imported monster will be
		hText = GetDlgItem(hdlg, IDC_TEXT_SIZE);
		if ((bmp.getWidth() <= 64) && (bmp.getHeight() <= 64))
		{
			SendMessage(hText, WM_SETTEXT, (WPARAM)0, (LPARAM)(LPCTSTR)"64x64");
		}
		else
		{
			SendMessage(hText, WM_SETTEXT, (WPARAM)0, (LPARAM)(LPCTSTR)"128x128");
		}
	}

	DeleteDC(bmpHdc);
	DeleteDC(picHdc);
	DeleteObject(tempBMP);
}

/*************************************/
//Redraws the paint buttons
void Tab1::drawPaintButtons(WPARAM wParam, LPARAM lParam)
{
	HBITMAP BMP;
	HDC bufferHdc;

	switch (((LPDRAWITEMSTRUCT)lParam)->CtlID)
	{
	case IDC_BUTTON_MONSTER_BRUSH:
		if (buttonSelected == 0)
			BMP = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(IDB_BRUSH2), IMAGE_BITMAP, 42, 42, LR_DEFAULTCOLOR);
		else
			BMP = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(IDB_BRUSH), IMAGE_BITMAP, 42, 42, LR_DEFAULTCOLOR);
		bufferHdc = CreateCompatibleDC(screenHdc);
		SelectObject(bufferHdc, BMP);
		BitBlt(((LPDRAWITEMSTRUCT)lParam)->hDC, 0, 0, 42, 42, bufferHdc, 0, 0, SRCCOPY);
		break;
	case IDC_BUTTON_MONSTER_ERASE:
		if (buttonSelected == 1)
			BMP = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(IDB_ERASE2), IMAGE_BITMAP, 42, 42, LR_DEFAULTCOLOR);
		else
			BMP = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(IDB_ERASE), IMAGE_BITMAP, 42, 42, LR_DEFAULTCOLOR);
		bufferHdc = CreateCompatibleDC(screenHdc);
		SelectObject(bufferHdc, BMP);
		BitBlt(((LPDRAWITEMSTRUCT)lParam)->hDC, 0, 0, 42, 42, bufferHdc, 0, 0, SRCCOPY);
		break;
	case IDC_BUTTON_MONSTER_GRAB:
		if (buttonSelected == 2)
			BMP = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(IDB_GRAB2), IMAGE_BITMAP, 42, 42, LR_DEFAULTCOLOR);
		else
			BMP = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(IDB_GRAB), IMAGE_BITMAP, 42, 42, LR_DEFAULTCOLOR);
		bufferHdc = CreateCompatibleDC(screenHdc);
		SelectObject(bufferHdc, BMP);
		BitBlt(((LPDRAWITEMSTRUCT)lParam)->hDC, 0, 0, 42, 42, bufferHdc, 0, 0, SRCCOPY);
		break;
	}
	DeleteDC(bufferHdc);
	DeleteObject(BMP);
}

/*************************************/
//Redraws the entire monster tab
void Tab1::drawTab(void)
{
	HDC picHdc;
	HDC bufferHdc;
	HWND hPic;
	HBITMAP bufferBMP;
	PAINTSTRUCT ps;
	RECT r, r2;
	int x, y, i, j;
	long color;
	int g;
	int maxY, maxX;

	if (!hTab)		//Exit if attempting to draw before the tab is fully initiliazed
		return;

	createHDCs();

	GetUpdateRect(hTab, &r, false);

	ps.fErase = false;
	ps.rcPaint = r;
	GetWindowRect(hTab, &r2);

	r.bottom += r2.top;		//Convert update rect to screen coordinates
	r.left += r2.left;
	r.right += r2.left;
	r.top += r2.top;

	//Draw monster pic
	hPic = GetDlgItem(hTab, IDC_PIC_MONSTER);

	GetWindowRect(hPic, &r2);

	if (overlapRect(r, r2))			//Check if monster pic needs to be updated
	{
		picHdc = GetDC(hPic);
		ps.hdc = picHdc;

		BeginPaint(hTab, &ps);

		BitBlt(picHdc, 0, 0, 128, 128, monsHdc, 0, 0, SRCCOPY);

		EndPaint(hTab, &ps);
		ReleaseDC(hPic,picHdc);
	}

	//Draw zoomed in monster editing pic
	hPic = GetDlgItem(hTab, IDC_PIC_EDITMONSTER);

	GetWindowRect(hPic, &r2);

	if (overlapRect(r, r2))			//Check if edit monster pic needs to be updated
	{
		picHdc = GetDC(hPic);
		ps.hdc = picHdc;

		BeginPaint(hTab, &ps);

		if ((monsters.getMonster(monsSelected))->getSize() == 0)
			maxY = 64 * magnification;
		else
			maxY = 128 * magnification;
		if (maxY > 256)
			maxY = 256;
		maxX = maxY;

		g = options.drawGrid(magnification);
		if (g == 1)		//Show pixel grid
		{
			for (i = 0; i <= maxY; i += magnification)
			{
				for (j = 0; j <= maxX; j++)
				{
					SetPixel(editHdc, i, j, 0x888888);
					SetPixel(editHdc, j, i, 0x888888);
				}
			}
		}
		if (g == 2)		//Show tile grid
		{
			int xoff, yoff;
			int xScroll, yScroll;

			xScroll = SendDlgItemMessage(hTab, IDC_HSCROLL_MONSTER_EDIT, SBM_GETPOS, 0, 0);
			yScroll = SendDlgItemMessage(hTab, IDC_VSCROLL_MONSTER_EDIT, SBM_GETPOS, 0, 0);

			xoff = (xScroll % 8) * magnification;
			yoff = (yScroll % 8) * magnification;

			for (i = 0; i <= maxY; i += magnification * 8)
			{
				for (j = 0; j <= maxX; j++)
				{
					x = i - xoff;
					if (x < 0)
						x += (256 - (256 % (magnification * 8))) + (magnification * 8);
					y = j;
					SetPixel(editHdc, x, y, 0x888888);
					y = i - yoff;
					if (y < 0)
						y += (256 - (256 % (magnification * 8))) + (magnification * 8);
					x = j;
					SetPixel(editHdc, x, y, 0x888888);
				}
			}
		}

		BitBlt(picHdc, 0, 0, 256, 256, editHdc, 0, 0, SRCCOPY);

		EndPaint(hTab, &ps);
		ReleaseDC(hPic, picHdc);
	}

	//Draw palette pic
	if ((monsters.getMonster(monsSelected))->getColors() == 8)
		palWidth = 36;
	else
		palWidth = 18;

	hPic = GetDlgItem(hTab, IDC_PIC_MONSTER_PALETTE);

	GetWindowRect(hPic, &r2);

	if (overlapRect(r, r2))			//Check if palette pic needs to be updated
	{
		picHdc = GetDC(hPic);
		ps.hdc = picHdc;

		//Create a buffer to draw the palette on
		bufferHdc = CreateCompatibleDC(screenHdc);
		bufferBMP = CreateCompatibleBitmap(screenHdc, 288, 30);
		SelectObject(bufferHdc, bufferBMP);

		//Draw the palette with a border around each color
		for (i = 0; i < (monsters.getMonster(monsSelected))->getColors(); i++)
		{
			color = RGBtoCOLORREF(monsters.monsPals[(monsters.getMonster(monsSelected))->getPalNum()][i]);
			for (y = 0; y < 30; y++)
			{
				if (palEntrySelected == i)
					SetPixel(bufferHdc, i * palWidth, y, 0xFFFFFF);
				else
					SetPixel(bufferHdc, i * palWidth, y, 0);
				for (x = 1; x < (palWidth - 1); x++)
				{
					if ((y == 0) || (y == 29))
						if (palEntrySelected == i)
							SetPixel(bufferHdc, x + i * palWidth, y, 0xFFFFFF);
						else
							SetPixel(bufferHdc, x + i * palWidth, y, 0);
					else
						SetPixel(bufferHdc, x + i * palWidth, y, color);
				}
				if (palEntrySelected == i)
					SetPixel(bufferHdc, x + i * palWidth, y, 0xFFFFFF);
				else
					SetPixel(bufferHdc, x + i * palWidth, y, 0);
			}
		}

		//Draw buffer to screen
		BeginPaint(hTab, &ps);

		BitBlt(picHdc, 0, 0, 288, 30, bufferHdc, 0, 0, SRCCOPY);

		EndPaint(hTab, &ps);
		ReleaseDC(hPic, picHdc);
		DeleteDC(bufferHdc);
		DeleteObject(bufferBMP);
	}

	updateControls();
}

/*************************************/
//Handles the action of the erase button
//Sets the pixel x,y of the monster to color 0 (transparent)
void Tab1::eraseButton(int x, int y)
{
	int max;

	m_changed = true;
	monsChanged = true;

	if ((monsters.getMonster(monsSelected))->getSize() == 0)
		max = 64;
	else max = 128;

	if ((x >= max) || (y >= max))
		return;

	(monsters.getMonster(monsSelected))->setPixel(x, y, 0);

	invalidatePics();
}

/*************************************/
void Tab1::exportImage(void)
{
	char filename[1024];
	int maxX, maxY;
	int fileType;

	sprintf(filename, "%s.png", (monsters.getMonster(monsSelected))->name);
	fileType = 2;

	if (exportDialog.show("Export monster", filename, &fileType))
	{
		(monsters.getMonster(monsSelected))->getDimensions(&maxX, &maxY);
		switch (fileType)
		{
		case 0:
			(monsters.getMonster(monsSelected))->pic.saveBMP(filename, maxX, maxY);
			break;
		case 1:
			(monsters.getMonster(monsSelected))->pic.savePCX(filename, maxX, maxY);
			break;
		case 2:
			(monsters.getMonster(monsSelected))->pic.savePNG(filename, maxX, maxY);
			break;
		}
	}
}

/*************************************/
void Tab1::flipImage(int flip)
{
	monsChanged = true;

	(monsters.getMonster(monsSelected))->flipImage(flip);
	invalidatePics();
}

/*************************************/
//Handles the scroll bars for the editing pic
void Tab1::handleMagScroll(int val, HWND hScroll, int pos)
{
	switch (val)
	{
	case SB_LINELEFT:
		pos = GetScrollPos(hScroll, SB_CTL) - 1;
		SetScrollPos(hScroll, SB_CTL, pos, true);
		break;
	case SB_LINERIGHT:
		pos = GetScrollPos(hScroll, SB_CTL) + 1;
		SetScrollPos(hScroll, SB_CTL, pos, true);
		break;
	case SB_PAGELEFT:
		pos = GetScrollPos(hScroll, SB_CTL) - 10;
		SetScrollPos(hScroll, SB_CTL, pos, true);
		break;
	case SB_PAGERIGHT:
		pos = GetScrollPos(hScroll, SB_CTL) + 10;
		SetScrollPos(hScroll, SB_CTL, pos, true);
		break;
	case SB_THUMBTRACK:
		SetScrollPos(hScroll, SB_CTL, pos, true);
	}
}

/*************************************/
//Determines where the mouse was clicked and handles any operations that need to be done
//Changes the palette selction if the palette bar is clicked on.
void Tab1::handleMouseClick(int x, int y)
{
	int tabx, taby;
	HWND hPal, hEdit;
	BYTE c;

	RECT r, r2;

	GetWindowRect(hTab, &r);

	tabx = r.left;
	taby = r.top;

	x += tabx;
	y += taby;

	//Check if the mouse was clicked on the palette bar

	hPal = GetDlgItem(hTab, IDC_PIC_MONSTER_PALETTE);
	GetWindowRect(hPal, &r);

	if (inRect(x, y, r))
	{
		changePalEntry((x - r.left) / palWidth);
	}

	//Check if the mouse was clicked on the editing pic and fill was selected

	hEdit = GetDlgItem(hTab, IDC_PIC_EDITMONSTER);
	GetWindowRect(hEdit, &r);

	if (inRect(x, y, r))
	{
		GetWindowRect(hEdit, &r2);

		//Find what pixel was clicked on
		y -= r2.top;
		x -= r2.left;

		x /= magnification;
		y /= magnification;

		x += SendDlgItemMessage(hTab, IDC_HSCROLL_MONSTER_EDIT, SBM_GETPOS, 0, 0);
		y += SendDlgItemMessage(hTab, IDC_VSCROLL_MONSTER_EDIT, SBM_GETPOS, 0, 0);

		if (buttonSelected == 0)
		{
			brushButton(x, y);
		}
		if (buttonSelected == 1)
		{
			eraseButton(x, y);
		}
		if (buttonSelected == 2)
		{
			c = (monsters.getMonster(monsSelected))->getPixel(x, y);

			changePalEntry(c);
		}
	}

}

/*************************************/
//Determines where the mouse is currently at
//Down is set if the left mouse button is pressed
//Handles updating the monster image, if the editing pic is clicked on
void Tab1::handleMouseMove(int x, int y, bool down)
{

	if (!down)			//Don't edit the pic unless the left mouse button is pressed
		return;

	int tabx, taby;
	HWND hEdit;
	BYTE c;
	RECT r, r2;

	GetWindowRect(hTab, &r);

	tabx = r.left;
	taby = r.top;

	x += tabx;
	y += taby;

	//Check if the mouse was clicked on the editing pic

	hEdit = GetDlgItem(hTab, IDC_PIC_EDITMONSTER);
	GetWindowRect(hEdit, &r);

	if (inRect(x, y, r))
	{
		GetWindowRect(hEdit, &r2);

		//Find what pixel was clicked on
		y -= r2.top;
		x -= r2.left;

		x /= magnification;
		y /= magnification;

		x += SendDlgItemMessage(hTab, IDC_HSCROLL_MONSTER_EDIT, SBM_GETPOS, 0, 0);
		y += SendDlgItemMessage(hTab, IDC_VSCROLL_MONSTER_EDIT, SBM_GETPOS, 0, 0);

		if (buttonSelected == 0)
		{
			brushButton(x, y);
		}
		if (buttonSelected == 1)
		{
			eraseButton(x, y);
		}
		if (buttonSelected == 2)
		{
			c = (monsters.getMonster(monsSelected))->getPixel(x, y);

			changePalEntry(c);
		}
	}
}

/*************************************/
//Handles functions for the palette sliders
void Tab1::handleSliders(WPARAM wParam, LPARAM lParam)
{
	HWND hSlider;
	BYTE val;

	monsChanged = true;
	m_changed = true;

	hSlider = GetDlgItem(hTab, IDC_SLIDER_MONSTER_RED);

	if ((HWND)lParam == hSlider)
	{
		val = (BYTE)SendMessage((HWND)lParam, TBM_GETPOS, 0, 0);
		monsters.monsPals[(monsters.getMonster(monsSelected))->getPalNum()][palEntrySelected].rgbtRed = val * 8;
		//Update Monster's bitmap 
		(monsters.getMonster(monsSelected))->changePalette((monsters.getMonster(monsSelected))->getPalNum());
		invalidatePics();
	}

	hSlider = GetDlgItem(hTab, IDC_SLIDER_MONSTER_GREEN);

	if ((HWND)lParam == hSlider)
	{
		val = (BYTE)SendMessage((HWND)lParam, TBM_GETPOS, 0, 0);
		monsters.monsPals[(monsters.getMonster(monsSelected))->getPalNum()][palEntrySelected].rgbtGreen = val * 8;
		//Update Monster's bitmap 
		(monsters.getMonster(monsSelected))->changePalette((monsters.getMonster(monsSelected))->getPalNum());
		invalidatePics();
	}

	hSlider = GetDlgItem(hTab, IDC_SLIDER_MONSTER_BLUE);

	if ((HWND)lParam == hSlider)
	{
		val = (BYTE)SendMessage((HWND)lParam, TBM_GETPOS, 0, 0);
		monsters.monsPals[(monsters.getMonster(monsSelected))->getPalNum()][palEntrySelected].rgbtBlue = val * 8;
		//Update Monster's bitmap 
		(monsters.getMonster(monsSelected))->changePalette((monsters.getMonster(monsSelected))->getPalNum());
		invalidatePics();
	}

}

/*************************************/
void Tab1::import(void)
{
	OPENFILENAME of;
	char type[10];
	char filename[256];
	char *title;

	if (tabSelected == 0)
		strcpy(type, "Monster");
	else
		strcpy(type, "Esper");

	title = (char *)malloc(50);
	strcpy(title, "Import ");

	ZeroMemory(filename, 256);

	of.Flags = OFN_ENABLETEMPLATE | OFN_EXPLORER | OFN_HIDEREADONLY
		| OFN_FILEMUSTEXIST | OFN_ENABLEHOOK;
	of.hInstance = hInst;
	of.hwndOwner = hTab;
	of.lCustData = 0;
	of.lpfnHook = ::tab1HookProc;
	of.lpstrCustomFilter = NULL;
	of.lpstrDefExt = NULL;
	of.lpstrFile = filename;
	of.lpstrFileTitle = NULL;
	of.lpstrFilter = "Supported images formats (*.png, *.bmp, *.pcx)\000*.png;*.bmp;*.pcx\000All files (*.*)\000*.*\000\000";
	of.lpstrInitialDir = NULL;
	of.lpstrTitle = strcat(title, type);
	of.lpTemplateName = MAKEINTRESOURCE(IDD_IMPORT_MONS);
	of.lStructSize = sizeof(OPENFILENAME);
	of.nFileExtension = 0;
	of.nFileOffset = 0;
	of.nFilterIndex = 0;
	of.nMaxCustFilter = 0;
	of.nMaxFile = 256;
	of.nMaxFileTitle = 256;

	GetOpenFileName(&of);

	m_changed = true;
}

/*************************************/
void Tab1::initTab(void)
{
	int i, found = -1;
	char temp[80];
	char temp2[80];

	monsBMP = CreateCompatibleBitmap(screenHdc, 128, 128);
	editBMP = CreateCompatibleBitmap(screenHdc, 256, 256);

	monsHdc = CreateCompatibleDC(screenHdc);
	editHdc = CreateCompatibleDC(screenHdc);

	SelectObject(monsHdc, monsBMP);
	SelectObject(editHdc, editBMP);

	buttonSelected = 0;

	addNames();

	//Add list of palettes
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_PALETTE, CB_RESETCONTENT, 0, 0);
	for (i = 0; i < 768; i++)
	{
		strcpy(temp, "Palette #");
		strcat(temp, _itoa(i, temp2, 10));
		SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_PALETTE, CB_ADDSTRING, 0,
			(LPARAM)(LPCTSTR)temp);
	}

	//Add list of sizes
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_SIZE, CB_RESETCONTENT, 0, 0);
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_SIZE, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)"64x64");
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_SIZE, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)"128x128");

	//Add list of colors
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_COLORS, CB_RESETCONTENT, 0, 0);
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_COLORS, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)"8 Colors");
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_COLORS, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)"16 Colors");

	changePalEntry(0);

	//Set ranges for color sliders
	SendDlgItemMessage(hTab, IDC_SLIDER_MONSTER_RED, TBM_SETRANGE, true, MAKELONG(0, 32));
	SendDlgItemMessage(hTab, IDC_SLIDER_MONSTER_GREEN, TBM_SETRANGE, true, MAKELONG(0, 32));
	SendDlgItemMessage(hTab, IDC_SLIDER_MONSTER_BLUE, TBM_SETRANGE, true, MAKELONG(0, 32));

	magnification = options.getTab1Mag();

	SendDlgItemMessage(hTab, IDC_UPDOWN_MONSTER_MAGNIFICATION, UDM_SETRANGE, 0, (LPARAM)MAKELONG((short)20, (short)2));
	SendDlgItemMessage(hTab, IDC_UPDOWN_MONSTER_MAGNIFICATION, UDM_SETPOS, 0, (LPARAM)MAKELONG((short)magnification, 0));

	createImageList();

	monsChanged = false;

	newSelection();
}

/*************************************/
//Invalidates the monster pic and the editing pic
void Tab1::invalidateMonsterPics(void)
{
	(monsters.getMonster(monsSelected))->createImage();		//Need to create image data to determine free space in ROM
	invalidateTabControl(IDC_PIC_MONSTER);
	invalidateTabControl(IDC_PIC_EDITMONSTER);
	drawTab();
}

/*************************************/
//Invalidates the monster pic,the editing pic, and the palette bar
//Calls drawTab to immediately update and redraw the tab
void Tab1::invalidatePics(void)
{
	invalidateTabControl(IDC_PIC_MONSTER);
	invalidateTabControl(IDC_PIC_EDITMONSTER);
	invalidateTabControl(IDC_PIC_MONSTER_PALETTE);

	(monsters.getMonster(monsSelected))->createImage();
	drawTab();
}

/*************************************/
//Creates a new blank image for the monster
void Tab1::newImage(void)
{
	char imageName[80] = "Image ";
	char temp[10];

	(monsters.getMonster(monsSelected))->newImage();
	strcat(imageName, _itoa(monsters.getNumImages(), temp, 10));
	strcat(imageName, " - ");
	strcat(imageName, (monsters.getMonster(monsSelected))->name);
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_IMAGE, CB_ADDSTRING, 0,
		(LPARAM)(LPCTSTR)imageName);
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_IMAGE, CB_SETCURSEL, monsters.getNumImages() - 1, 0);
	monsChanged = true;
	createImageList();
	invalidatePics();
}

/*************************************/
//Creates a new image list and name list when you open a new ROM
//Also redraws the tab
void Tab1::newROM(void)
{
	addNames();
	createImageList();
	invalidateTab();
}

/*************************************/
//Handles the selection of a new monster
//updates the combo boxes, redraws the images, and palettes, etc.
void Tab1::newSelection(void)
{
	int i;

	if (monsChanged)
	{
		monsChanged = false;
		(monsters.getMonster(monsSelected))->createImage();
		monsters.updateMonsters();
	}

	monsSelected = SendDlgItemMessage(hTab, IDC_COMBO_MONSTER, CB_GETCURSEL, 0, 0);
	if (tabSelected == 1)
	{
		monsSelected += 384;			//Skip monsters and select Espers
		if (monsSelected > 399)			//Skip Ragnarok
			monsSelected++;
	}
	i = (monsters.getMonster(monsSelected))->getSize();
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_SIZE, CB_SETCURSEL, i, 0);
	i = ((monsters.getMonster(monsSelected))->getColors() == 16);
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_COLORS, CB_SETCURSEL, i, 0);
	i = (monsters.getMonster(monsSelected))->getPalNum();
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_PALETTE, CB_SETCURSEL, i, 0);
	i = (monsters.getMonster(monsSelected))->getImageNum();
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_IMAGE, CB_SETCURSEL, i, 0);

	invalidatePics();
}


/*************************************/
//Saves the monster data to the ROM
bool Tab1::save_internal(char *filename)
{
	if (monsChanged)
	{
		monsChanged = false;
	}
	(monsters.getMonster(monsSelected))->createImage();
	monsters.updateMonsters();

	monsters.compress();

	if (monsters.saveMonsters(filename))
	{
		monsters.loadMonsters(filename);

		createImageList();
		updateControls();

		return true;
	}

	return false;
}

/*************************************/
//Selected the nth paint button
void Tab1::selectButton(BYTE n)
{
	buttonSelected = n;
	invalidateTab();
}

/*************************************/
//If the mouse is over the editing pic, changes the cursor to the appropriate cursor
void Tab1::setCursor(void)
{
	POINT mousePos;
	HWND hEdit;
	RECT r;

	GetCursorPos(&mousePos);

	hEdit = GetDlgItem(hTab, IDC_PIC_EDITMONSTER);
	GetWindowRect(hEdit, &r);

	if (inRect(mousePos.x, mousePos.y, r))
	{
		switch (buttonSelected)
		{
		case 0:
			SetCursor(brushCursor);
			break;
		case 1:
			SetCursor(eraseCursor);
			break;
		case 2:
			SetCursor(grabCursor);
			break;
		}
	}
	else
		SetCursor(LoadCursor(NULL, IDC_ARROW));
}

/*************************************/
//If the selected palette entry is 0 disables the palette sliders, otherwise enables them
void Tab1::sliderEnable(void)
{
	HWND hSlider;

	if (palEntrySelected == 0)
	{
		hSlider = GetDlgItem(hTab, IDC_SLIDER_MONSTER_RED);
		EnableWindow(hSlider, false);
		hSlider = GetDlgItem(hTab, IDC_SLIDER_MONSTER_GREEN);
		EnableWindow(hSlider, false);
		hSlider = GetDlgItem(hTab, IDC_SLIDER_MONSTER_BLUE);
		EnableWindow(hSlider, false);
		hSlider = GetDlgItem(hTab, IDC_CHECK_MONSTER_TRANSPARENT);
		EnableWindow(hSlider, false);
		SendMessage(hSlider, BM_SETCHECK, (WPARAM)BST_CHECKED, 0);
	}
	else
	{
		hSlider = GetDlgItem(hTab, IDC_SLIDER_MONSTER_RED);
		EnableWindow(hSlider, true);
		hSlider = GetDlgItem(hTab, IDC_SLIDER_MONSTER_GREEN);
		EnableWindow(hSlider, true);
		hSlider = GetDlgItem(hTab, IDC_SLIDER_MONSTER_BLUE);
		EnableWindow(hSlider, true);
		hSlider = GetDlgItem(hTab, IDC_CHECK_MONSTER_TRANSPARENT);
		EnableWindow(hSlider, true);
		SendMessage(hSlider, BM_SETCHECK, (WPARAM)BST_UNCHECKED, 0);
	}
}

/*************************************/
//Switches to Monster or Esper Tab based on the Tab selected
void Tab1::switchType(void)
{
	addNames();

	newSelection();
}

/*************************************/
//Updates all the combo boxes, sliders, etc.
void Tab1::updateControls(void)
{
	char temp[10];
	char mold[10] = "Mold: ";
	char free[40] = "Free Space: ";
	char mag[20];
	int maxScroll;

	HWND hPic;

	int i;

	SendDlgItemMessage(hTab, IDC_SLIDER_MONSTER_RED, TBM_SETPOS, true,
		monsters.monsPals[(monsters.getMonster(monsSelected))->getPalNum()][palEntrySelected].rgbtRed / 8);
	SendDlgItemMessage(hTab, IDC_SLIDER_MONSTER_GREEN, TBM_SETPOS, true,
		monsters.monsPals[(monsters.getMonster(monsSelected))->getPalNum()][palEntrySelected].rgbtGreen / 8);
	SendDlgItemMessage(hTab, IDC_SLIDER_MONSTER_BLUE, TBM_SETPOS, true,
		monsters.monsPals[(monsters.getMonster(monsSelected))->getPalNum()][palEntrySelected].rgbtBlue / 8);

	SendDlgItemMessage(hTab, IDC_TEXT_MONSTER_REDVAL, WM_SETTEXT, (WPARAM)0,
		(LPARAM)(LPCTSTR)_itoa(monsters.monsPals[(monsters.getMonster(monsSelected))->getPalNum()][palEntrySelected].rgbtRed / 8,
			temp, 10));
	SendDlgItemMessage(hTab, IDC_TEXT_MONSTER_GREENVAL, WM_SETTEXT, (WPARAM)0,
		(LPARAM)(LPCTSTR)_itoa(monsters.monsPals[(monsters.getMonster(monsSelected))->getPalNum()][palEntrySelected].rgbtGreen / 8,
			temp, 10));
	SendDlgItemMessage(hTab, IDC_TEXT_MONSTER_BLUEVAL, WM_SETTEXT, (WPARAM)0,
		(LPARAM)(LPCTSTR)_itoa(monsters.monsPals[(monsters.getMonster(monsSelected))->getPalNum()][palEntrySelected].rgbtBlue / 8,
			temp, 10));

	i = (monsters.getMonster(monsSelected))->getMoldNum();
	_itoa(i, temp, 10);
	strcat(mold, temp);
	SendDlgItemMessage(hTab, IDC_TEXT_MONSTER_MOLD, WM_SETTEXT, 0, (LPARAM)(LPCTSTR)(mold));

	i = (monsters.getMonster(monsSelected))->getSize();
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_SIZE, CB_SETCURSEL, i, 0);
	i = ((monsters.getMonster(monsSelected))->getColors() == 16);
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_COLORS, CB_SETCURSEL, i, 0);
	i = (monsters.getMonster(monsSelected))->getPalNum();
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_PALETTE, CB_SETCURSEL, i, 0);
	i = (monsters.getMonster(monsSelected))->getImageNum();
	SendDlgItemMessage(hTab, IDC_COMBO_MONSTER_IMAGE, CB_SETCURSEL, i, 0);

	i = monsters.getFreeRoom();

	if (i < 0)
	{
		hPic = GetDlgItem(hTab, IDC_PIC_MONSTER_NOFREESPACE);
		ShowWindow(hPic, SW_NORMAL);
		hPic = GetDlgItem(hTab, IDC_PIC_MONSTER_FREESPACE);
		ShowWindow(hPic, SW_HIDE);
	}
	else
	{
		hPic = GetDlgItem(hTab, IDC_PIC_MONSTER_FREESPACE);
		ShowWindow(hPic, SW_NORMAL);
		hPic = GetDlgItem(hTab, IDC_PIC_MONSTER_NOFREESPACE);
		ShowWindow(hPic, SW_HIDE);
	}


	_itoa(i, temp, 10);
	strcat(free, temp);
	SendDlgItemMessage(hTab, IDC_TEXT_MONSTER_FREESPACE, WM_SETTEXT, 0, (LPARAM)(LPCTSTR)(free));

	magnification = (BYTE)SendDlgItemMessage(hTab, IDC_UPDOWN_MONSTER_MAGNIFICATION, UDM_GETPOS, 0, 0);

	if (magnification == 0)
		magnification = 1;

	options.setTab1Mag(magnification);

	sprintf(mag, "magnification: x%d", magnification);
	SendDlgItemMessage(hTab, IDC_TEXT_MONSTER_MAGNIFICATION, WM_SETTEXT, 0, (LPARAM)(LPCTSTR)(mag));

	if ((monsters.getMonster(monsSelected))->getSize() == 0)
		maxScroll = 64 - (256 / magnification);
	else
		maxScroll = 128 - (256 / magnification);

	if (maxScroll < 0)
	{
		maxScroll = 0;
	}

	HWND hScroll = GetDlgItem(hTab, IDC_HSCROLL_MONSTER_EDIT);
	SetScrollRange(hScroll, SB_CTL, 0, maxScroll, true);
	if (maxScroll > 0)
		ShowWindow(hScroll, true);
	else
		ShowWindow(hScroll, false);

	hScroll = GetDlgItem(hTab, IDC_VSCROLL_MONSTER_EDIT);
	SetScrollRange(hScroll, SB_CTL, 0, maxScroll, true);
	if (maxScroll > 0)
		ShowWindow(hScroll, true);
	else
		ShowWindow(hScroll, false);
}

/*************************************/
BOOL Tab1::tabProc(HWND hdlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	HWND hScroll;

	switch (message)
	{
	case WM_CLOSE:
		hTab = NULL;
		break;
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_BUTTON_EXIT:
			SendMessage(hWnd, WM_CLOSE, 0, 0);
			return true;
			break;
		case IDC_BUTTON_MONSTER_IMPORT:
			import();
			return true;
			break;
		case IDC_BUTTON_MONSTER_NEW:
			newImage();
			return true;
			break;
		case IDC_BUTTON_MONSTER_COPY:
			copyImage();
			return true;
			break;
		case IDC_BUTTON_MONSTER_CLEAR:
			clearImage();
			return true;
			break;
		case IDC_COMBO_MONSTER:
			if (HIWORD(wParam) == CBN_SELCHANGE)
			{
				newSelection();
			}
			return true;
			break;
		case IDC_COMBO_MONSTER_IMAGE:
			if (HIWORD(wParam) == CBN_SELCHANGE)
			{
				changeImage();
			}
			return true;
			break;
		case IDC_COMBO_MONSTER_PALETTE:
			if (HIWORD(wParam) == CBN_SELCHANGE)
			{
				changePalette();
			}
			return true;
			break;
		case IDC_COMBO_MONSTER_SIZE:
			if (HIWORD(wParam) == CBN_SELCHANGE)
			{
				changeSize();
			}
			return true;
			break;
		case IDC_COMBO_MONSTER_COLORS:
			if (HIWORD(wParam) == CBN_SELCHANGE)
			{
				changeNumColors();
			}
			return true;
			break;
		case IDC_BUTTON_MONSTER_BRUSH:
			if (HIWORD(wParam) == BN_CLICKED)
			{
				selectButton(0);
			}
			return true;
			break;
		case IDC_BUTTON_MONSTER_ERASE:
			if (HIWORD(wParam) == BN_CLICKED)
			{
				selectButton(1);
			}
			return true;
			break;
		case IDC_BUTTON_MONSTER_GRAB:
			if (HIWORD(wParam) == BN_CLICKED)
			{
				selectButton(2);
			}
			return true;
			break;
		case IDC_CHECK_MONSTER_TRANSPARENT:
			if (HIWORD(wParam) == BN_CLICKED)
			{
				changeTransparency();
			}
			return true;
			break;
		case IDC_UPDOWN_MONSTER_MAGNIFICATION:
			invalidatePics();
			return true;
			break;
		}
		break;
	case WM_DRAWITEM:
		drawPaintButtons(wParam, lParam);
		break;
	case WM_LBUTTONDOWN:
		handleMouseClick(LOWORD(lParam), HIWORD(lParam));
		return true;
		break;
	case WM_VSCROLL:
		hScroll = GetDlgItem(hTab, IDC_VSCROLL_MONSTER_EDIT);
		if ((HWND)lParam == hScroll)
		{
			handleMagScroll(LOWORD(wParam), (HWND)lParam, HIWORD(wParam));
		}
		invalidatePics();
		return false;
		break;
	case  WM_HSCROLL:
		hScroll = GetDlgItem(hTab, IDC_HSCROLL_MONSTER_EDIT);
		if ((HWND)lParam == hScroll)
		{
			handleMagScroll(LOWORD(wParam), (HWND)lParam, HIWORD(wParam));
			invalidatePics();
		}
		else
		{
			handleSliders(wParam, lParam);
		}
		return false;
		break;
	case WM_MOUSEMOVE:
		handleMouseMove(LOWORD(lParam), HIWORD(lParam), wParam & MK_LBUTTON);
		return true;
		break;
	case WM_PAINT:
		drawTab();
		return false;
		break;
	case WM_SETCURSOR:
		setCursor();
		return true;
		break;
	}
	return false;
}

/*************************************/
UINT Tab1::tab1HookProc(HWND hdlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	if (message == WM_NOTIFY)
	{
		switch (((LPOFNOTIFY)lParam)->hdr.code)
		{
		case CDN_SELCHANGE:
			displayPreview(hdlg);
			return 1;
			break;
		case CDN_FILEOK:
			Bitmap newBMP;

			switch (newBMP.loadFile(((LPOFNOTIFY)lParam)->lpOFN->lpstrFile, 256, 200, 200))
			{
			case 0:
				(monsters.getMonster(monsSelected))->setBitmap(newBMP);
				(monsters.getMonster(monsSelected))->getNewMold();
				invalidatePics();
				monsChanged = true;
				return 0;
				break;
			case 1:
				MessageBox(NULL, "Image has more than 16 colors", ff3seTitle, MB_OK | MB_ICONERROR | MB_TASKMODAL);
				break;
			case 2:
				MessageBox(NULL, "Image is larger than 128x128", ff3seTitle, MB_OK | MB_ICONERROR | MB_TASKMODAL);
				break;
			case 3:
				MessageBox(NULL, "Error opening file", ff3seTitle, MB_OK | MB_ICONERROR | MB_TASKMODAL);
				break;
			case 4:
				MessageBox(NULL, "File does not appear to be a valid image file", ff3seTitle, MB_OK | MB_ICONERROR | MB_TASKMODAL);
				break;
			}
			SetWindowLong(hdlg, DWL_MSGRESULT, 1);
			return 1;
			break;
		}
	}
	return 0;
}

/*************************************/
bool Tab1::copyImageProc(HWND hdlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	if (message == WM_COMMAND)
	{
		switch (LOWORD(wParam))
		{
		case IDC_COPY_LIST:
			if (HIWORD(wParam) != LBN_DBLCLK)
				break;
		case IDC_COPY_OK:
			HWND hList;
			int image;

			hList = GetDlgItem(hdlg, IDC_COPY_LIST);
			image = SendDlgItemMessage(hdlg, IDC_COPY_LIST, LB_GETCURSEL, 0, 0);

			(monsters.getMonster(monsSelected))->copyImage(image);

			invalidatePics();

			BringWindowToTop(hWnd);
			EnableWindow(hWnd, true);
			DestroyWindow(hdlg);
			return true;
			break;
		case IDC_COPY_CANCEL:
			BringWindowToTop(hWnd);
			EnableWindow(hWnd, true);
			DestroyWindow(hdlg);
			hDlg = NULL;
			return true;
			break;
		}
	}
	return false;
}

/*************************************/
UINT CALLBACK tab1HookProc(HWND hdlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	return tab1.tab1HookProc(hdlg, message, wParam, lParam);
}

/*************************************/
BOOL CALLBACK copyMonsterProc(HWND hdlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	return tab1.copyImageProc(hdlg, message, wParam, lParam);
}

/*************************************/
BOOL CALLBACK tab1Proc(HWND hdlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	return tab1.tabProc(hdlg, message, wParam, lParam);
}

